/*
 * MT.h
 *
 * public header for shared library
 *
 * This wizard-generated code is based on code adapted from the
 * SampleLib project distributed as part of the Palm OS SDK 4.0,
 *
 * Copyright (c) 1994-1999 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 */

#ifndef MT_H_
#define MT_H_

/* Palm OS common definitions */
#include <SystemMgr.h>

#include "fixed.h"
#include "layer3.h"

/* If we're actually compiling the library code, then we need to
 * eliminate the trap glue that would otherwise be generated from
 * this header file in order to prevent compiler errors in CW Pro 2. */
#ifdef BUILDING_MT
	#define MT_LIB_TRAP(trapNum)
#else
	#define MT_LIB_TRAP(trapNum) SYS_TRAP(trapNum)
#endif

/*********************************************************************
 * Type and creator of Sample Library database
 *********************************************************************/

#define		MTCreatorID	'hSmt'
#define		MTTypeID		sysFileTLibrary

/*********************************************************************
 * Internal library name which can be passed to SysLibFind()
 *********************************************************************/

#define		MTName		"HS_MP3_Tables"

/*********************************************************************
 * HS_MP3_Tables result codes
 * (appErrorClass is reserved for 3rd party apps/libraries.
 * It is defined in SystemMgr.h)
 *********************************************************************/

/* invalid parameter */
#define MTErrParam		(appErrorClass | 1)		

/* library is not open */
#define MTErrNotOpen		(appErrorClass | 2)		

/* returned from MTClose() if the library is still open */
#define MTErrStillOpen	(appErrorClass | 3)		

/*********************************************************************
 * API Prototypes
 *********************************************************************/

typedef struct mad_layer3_tables MTTablesType, *MTTablesPtr;

#ifdef __cplusplus
extern "C" {
#endif

/* Standard library open, close, sleep and wake functions */

extern Err MTOpen(UInt16 refNum, UInt32 * clientContextP)
	MT_LIB_TRAP(sysLibTrapOpen);
				
extern Err MTClose(UInt16 refNum, UInt32 clientContext)
	MT_LIB_TRAP(sysLibTrapClose);

extern Err MTSleep(UInt16 refNum)
	MT_LIB_TRAP(sysLibTrapSleep);

extern Err MTWake(UInt16 refNum)
	MT_LIB_TRAP(sysLibTrapWake);

/* Custom library API functions */

extern Err MTGetTablesLE(UInt16 refNum, MTTablesPtr tablesP)
	MT_LIB_TRAP(sysLibTrapBase + 5);

extern Err MTGetTablesBE(UInt16 refNum, MTTablesPtr tablesP)
	MT_LIB_TRAP(sysLibTrapBase + 6);

extern Err MTFreeTables(UInt16 refNum, MTTablesPtr tablesP)
	MT_LIB_TRAP(sysLibTrapBase + 7);

#ifdef __cplusplus
}
#endif

/*
 * FUNCTION: MT_OpenLibrary
 *
 * DESCRIPTION:
 *
 * User-level call to open the library.  This inline function
 * handles the messy task of finding or loading the library
 * and calling its open function, including handling cleanup
 * if the library could not be opened.
 * 
 * PARAMETERS:
 *
 * refNumP
 *		Pointer to UInt16 variable that will hold the new
 *      library reference number for use in later calls
 *
 * clientContextP
 *		pointer to variable for returning client context.  The client context is
 *		used to maintain client-specific data for multiple client support.  The 
 *		value returned here will be used as a parameter for other library 
 *		functions which require a client context.  
 *
 * CALLED BY: System
 *
 * RETURNS:
 *		errNone
 *		memErrNotEnoughSpace
 *      sysErrLibNotFound
 *      sysErrNoFreeRAM
 *      sysErrNoFreeLibSlots
 *
 * SIDE EFFECTS:
 *		*clientContextP will be set to client context on success, or zero on
 *      error.
 */
 
__inline Err MT_OpenLibrary(UInt16 *refNumP, UInt32 * clientContextP)
{
	Err error;
	Boolean loaded = false;
	
	/* first try to find the library */
	error = SysLibFind(MTName, refNumP);
	
	/* If not found, load the library instead */
	if (error == sysErrLibNotFound)
	{
		error = SysLibLoad(MTTypeID, MTCreatorID, refNumP);
		loaded = true;
	}
	
	if (error == errNone)
	{
		error = MTOpen(*refNumP, clientContextP);
		if (error != errNone)
		{
			if (loaded)
			{
				SysLibRemove(*refNumP);
			}

			*refNumP = sysInvalidRefNum;
		}
	}
	
	return error;
}

/*
 * FUNCTION: MT_CloseLibrary
 *
 * DESCRIPTION:	
 *
 * User-level call to closes the shared library.  This handles removal
 * of the library from system if there are no users remaining.
 *
 * PARAMETERS:
 *
 * refNum
 *		Library reference number obtained from MT_OpenLibrary().
 *
 * clientContext
 *		client context (as returned by the open call)
 *
 * CALLED BY: Whoever wants to close the library
 *
 * RETURNS:
 *		errNone
 *		sysErrParamErr
 */

__inline Err MT_CloseLibrary(UInt16 refNum, UInt32 clientContext)
{
	Err error;
	
	if (refNum == sysInvalidRefNum)
	{
		return sysErrParamErr;
	}

	error = MTClose(refNum, clientContext);

	if (error == errNone)
	{
		/* no users left, so unload library */
		SysLibRemove(refNum);
	} 
	else if (error == MTErrStillOpen)
	{
		/* don't unload library, but mask "still open" from caller  */
		error = errNone;
	}
	
	return error;
}

#endif /* MT_H_ */
